using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace GameOfLife
{
    class LifeField
    {
        public struct myByte
        {
            public Byte bits;  // bits in this byte
            public int[] age; // age of each bit
        }

        public struct PointColor
        {
            public Color color;
            public Vector2 point;
        }

        #region private variables

        // our width and height
        int m_width;
        int m_height;

        // a boolean to switch between buffers
        Boolean useField1ForSrc;

        // set to true while update() is being called;
        Boolean inTransition;  

        // our data
        myByte[] m_field1;
        myByte[] m_field2;

        // the bitmap of ourself
        Color[] m_bitmap;

        #endregion

        /// <summary>
        /// color of a dead node
        /// </summary>
        Color colorWhenDead;
        /// <summary>
        /// color of a living node
        /// </summary>
        Color colorWhenAlive;

        public LifeField(int width, int height)
        {
            m_width = width;
            m_height = height;
            useField1ForSrc = false;
            inTransition = false;
            m_field1 = new myByte[width / 8 * height];
            m_field2 = new myByte[width / 8 * height];
            for(int i = 0; i < m_field1.Length; ++i)
            {
                m_field1[i].bits = (Byte)0x00;
                m_field1[i].age = new int[8];
                m_field2[i].bits = (Byte)0x00;
                m_field2[i].age = new int[8];
            }

            colorWhenDead = Color.Gray;
            colorWhenAlive = Color.Black;
            m_bitmap = new Color[width * height];
        }

        public void ClearField()
        {
            for (int i = 0; i < m_field1.Length; ++i)
            {
                m_field1[i].bits = (Byte)0x00;
                m_field1[i].age = new int[8];
                m_field2[i].bits = (Byte)0x00;
                m_field2[i].age = new int[8];
            }

            for (int i = 0; i < m_bitmap.Length; ++i)
                m_bitmap[i] = colorWhenDead;
        }

        public Boolean this[int x, int y]
        {
            get
            {
                if (x < 0 || x >= m_width)
                    return false;
                if (y < 0 || y >= m_height)
                    return false;

                if (useField1ForSrc && inTransition)
                {
                    int bIndex = (y * m_width + x) / 8;
                    myByte b = m_field1[bIndex];
                    Byte result = (Byte)(b.bits & (0x01 << (0x07 - x % 8)));
                    return result != 0;
                }
                else
                {
                    int bIndex = (y * m_width + x) / 8;
                    myByte b = m_field2[bIndex];
                    Byte result = (Byte)(b.bits & (0x01 << (0x07 - x % 8)));
                    return result != 0;
                }
            }
            set
            {
                if (x < 0 || x >= m_width)
                    return;
                if (y < 0 || y >= m_height)
                    return;

                // if field1 is the source, then we WRITE to field 2
                if (useField1ForSrc)
                {
                    int bIndex = (y * m_width + x) / 8;
                    if (value == true)
                    {
                        // if we're already alive
                        if ((m_field1[bIndex].bits & (0x01 << (0x07 - x % 8))) != 0)
                            // then increment our age
                            m_field2[bIndex].age[x % 8] = m_field1[bIndex].age[x % 8] + 1;
                        else
                            // else, we're becoming alive, and our age is 0
                            m_field2[bIndex].age[x % 8] = 0;
                        m_field2[bIndex].bits |= (Byte)((Byte)0x01 << (0x07 - x % 8));

                        // add to drawable points
                        m_bitmap[(y * m_width + x)] = colorWhenAlive;
                    }
                    else
                    {
                        m_field2[bIndex].age[x % 8] = 0;
                        m_field2[bIndex].bits &= (Byte)(0xff - (0x01 << (0x07 - x % 8)));

                        m_bitmap[(y * m_width + x)] = colorWhenDead;
                    }
                }
                else // and vice versa
                {
                    int bIndex = (y * m_width + x) / 8;
                    if (value == true)
                    {
                        // if we're already alive
                        if ((m_field2[bIndex].bits & (0x01 << (0x07 - x % 8))) != 0)
                            // then increment our age
                            m_field1[bIndex].age[x % 8] = m_field2[bIndex].age[x % 8] + 1;
                        else
                            // else, we're becoming alive, and our age is 0
                            m_field1[bIndex].age[x % 8] = 0;
                        m_field1[bIndex].bits |= (Byte)((Byte)0x01 << (0x07 - x % 8));

                        // add to drawable points
                        m_bitmap[(y * m_width + x)] = colorWhenAlive;
                    }
                    else
                    {
                        m_field1[bIndex].age[x % 8] = 0;
                        m_field1[bIndex].bits &= (Byte)(0xff - (0x01 << (0x07 - x % 8)));

                        m_bitmap[(y * m_width + x)] = colorWhenDead;
                    }
                }
            }
        }

        public int GetAge(int x, int y)
        {
            if (x < 0 || x >= m_width)
                return 0;
            if (y < 0 || y >= m_height)
                return 0;

            int bIndex = (y * m_width + x) / 8;
            if(useField1ForSrc)
                return m_field1[bIndex].age[x % 8];
            else
                return m_field2[bIndex].age[x % 8];
        }

        public void Update(GameTime gameTime)
        {
            useField1ForSrc = !useField1ForSrc;

            // begin calculating new field.
            inTransition = true;
            for(int i = 0; i < m_bitmap.Length; ++i)
                m_bitmap[i] = colorWhenDead;

            Boolean myValue;
            int livingNeighbors;
            for (int y = 0; y < m_height; ++y)
            {

                for (int x = 0; x < m_width; ++x)
                {
                    livingNeighbors = 0;
                    myValue = this[x, y];

                    livingNeighbors += this[x - 1, y - 1] ? 1 : 0;
                    livingNeighbors += this[x - 1, y] ? 1 : 0;
                    livingNeighbors += this[x - 1, y + 1] ? 1 : 0;

                    livingNeighbors += this[x, y - 1] ? 1 : 0;
                    livingNeighbors += this[x, y + 1] ? 1 : 0;

                    livingNeighbors += this[x + 1, y - 1] ? 1 : 0;
                    livingNeighbors += this[x + 1, y] ? 1 : 0;
                    livingNeighbors += this[x + 1, y + 1] ? 1 : 0;

                    if (((livingNeighbors == 2 || livingNeighbors == 3) && myValue == true) ||
                        myValue == false && livingNeighbors == 3)
                    {
                        this[x, y] = true;
                        int age = GetAge(x, y);
                    }
                    else
                    {
                        this[x, y] = false;
                    }
                }
            }

            // finished calculating new field.
            inTransition = false;
        }

        public Color[] GetColorMap()
        {
            return m_bitmap;
        }
    }
}
